<Shader Name="ParticlesRender">
	<Uniforms>
		<Texture Name="AlbedoTexture" Uniform="AlbedoTexture"/>
		<Vector4 Name="AlbedoColor" Uniform="AlbedoColor"/>
		<Texture Name="NormalTexture" Uniform="NormalTexture" IsLinear="True"/>
		<Float Name="Roughness" Uniform="Roughness"/>
		<Float Name="Metalness" Uniform="Metalness"/>
		
		<Texture Name="EmissionTexture" Uniform="EmissionTexture"/>
		<Vector4 Name="EmissionColor" Uniform="EmissionColor" IsLinear="True"/>
		<Float Name="Strength" Uniform="Strength"/>
		
		<Texture Name="SSS_ColorTexture" Uniform="SSS_ColorTexture"/>
		<Vector4 Name="SSS_ColorColor" Uniform="SSS_ColorColor"/>
		<Float Name="SSS_Amount" Uniform="SSS_Amount"/>
		<Float Name="SSS_Thickness" Uniform="SSS_Thickness"/>
	</Uniforms>
	<Passes>
		<Pass Name="Deferred">
			<Vertex>
				#version 430 core
				#define VERTEX_SHADER
				#include "HLSL"
				#include "Random"
				 
				in float3 position;
				in float3 normal;
				out float4 INscreenPos;
				out float4 INworldPos;
				out float4 INPscreenPos;
				out float3 INworldNor;

				void main(){
					gl_Position = _WorldToView*float4(position, 1.0);
					INworldPos = float4(position, 1.0);
					INscreenPos=_WorldToView*float4(position, 1.0);
					INPscreenPos = _WorldToView_Past*float4(normal, 1.0);
					INworldNor = float3(0,0,1);//normalize((_LocalToWorldNormal*float3(normal)).xyz);
					
					
					//Temporal Anti-aliasing Jitter
					float time = mod(Time.w,4);//(mod(Time.w,2)==1)?1:0;
					gl_Position.xy += (((vec2(Random(time),Random(time-138.4))))-0.5)*2*(ScreenSize.zw)*gl_Position.w;
					
					//Scale points by distance
   					gl_PointSize = 42 * tan(72.196567934 / 2) * 0.025 / gl_Position.w*(ScreenSize.x/1920.0);
				}
			</Vertex>
			<Fragment><![CDATA[
				#version 330 core
				#include "HLSL"
				#line 27
				
				in float4 INscreenPos;
				in float4 INworldPos;
				in float4 INPscreenPos;
				in float3 INworldNor;
				
				layout(location = 0) out float3 Normals;
				layout(location = 1) out float2 MotionBuffer;
				
				void main(){
					//Big hacky sphere thingy
					vec2 circCoord = 2.0 * gl_PointCoord - 1.0;
					if (dot(circCoord, circCoord) > 1.0) {
						discard;
					}
					float3 worldNor = normalize(_CameraPos-INworldPos.xyz);
					float len = length(circCoord);
					vec3 normal = normalize ( vec3 (circCoord,  cos ( len*3.1415/  2) ) ) ;
					worldNor = normalize(lerp(worldNor,-(transpose(_WorldToView)*float4(normal,0.0)).xyz,1));
					Normals = worldNor*0.5+0.5;
					
					vec3 screenPos = (INscreenPos.xyz/INscreenPos.w+1.0)/2.0;
					vec3 PscreenPos = (INPscreenPos.xyz/INPscreenPos.w+1.0)/2.0;
					MotionBuffer = (screenPos.xy-PscreenPos.xy);
				}
			]]></Fragment>
		</Pass>
		<Pass Name="VoxelizeDetail">
			<Vertex><![CDATA[
				#version 330 core
				#define VERTEX_SHADER
				#include "HLSL"
				uniform mat4 _LightFieldWorldToLocal;
				 
				in float3 position;
				in float3 normal;
				out float3 INworldPos;
				uniform float Strength;



				void main(){
					float3 movpos = position;
					INworldPos = movpos.xyz;
					gl_Position = float4(movpos,1.0);
					gl_Position = vec4((((_LightFieldWorldToLocal*vec4(position.xyz,1))).xyz*2.f).xyz,1.0);
					gl_PointSize = 1;
				}
			]]></Vertex>
			<Fragment><![CDATA[
				#version 430 core
				#include "HLSL"
				#include "LightFields"
				#line 601
				layout (location = 0,r8) uniform restrict image3D _RayTexImageAlp;
				
				in float3 INworldPos;
				void main(){
					float3 voxelPos = INworldPos;
					voxelPos = (_LightFieldWorldToLocal*float4(voxelPos,1)).xyz;
					voxelPos+=0.5;
					voxelPos*=_LightFieldDistTexSize;
					voxelPos.z+=1;//Push the balls above ground plane voxels

					imageStore(_RayTexImageAlp, ivec3(voxelPos), float4(voxelPhase));
					return;
				}
			]]></Fragment>
		</Pass>
	
	
	
		<Pass Name="VoxelizeMidAndColor">
			<Vertex><![CDATA[
				#version 430 core
				#define VERTEX_SHADER
				#include "HLSL"
				uniform mat4 _LightFieldWorldToLocal;
				 
				in float3 position;
				in float3 normal;
				in float2 uv;
				out float3 INworldPos;
				out float3 INVPworldPos;
				out float3 INworldNor;
				out float4 INVscreenPos;
				out float4 INVPscreenPos;
				out float2 INVuv;
				uniform float Strength;



				void main(){
					float3 movpos = position;
					
					INworldPos = movpos.xyz;
					INworldNor = normalize((_LocalToWorldNormal*normal).xyz);
					gl_Position = float4(movpos,1.0);//2.0;
					gl_Position = vec4((((_LightFieldWorldToLocal*vec4(position.xyz,1))).xyz*2.f).xyz,1.0);
					gl_PointSize = 1;
				}
			]]></Vertex>
			<Fragment><![CDATA[
				#version 430 core
				#include "HLSL"
				#include "LightFields"
				#line 601
				uniform float4 AlbedoColor;
				uniform float4 EmissionColor;
				uniform float Strength;
				uniform sampler2D AlbedoTexture;
				uniform sampler2D TempLightmapStopgap;
				uniform sampler3D _RayTex;
				layout (location = 0,rgba16f) uniform restrict image3D _RayTexImage;
				layout (location = 2,rgba16f) uniform restrict image3D _RayTexImageM2;
				
				layout (location = 4,r8) uniform restrict image3D _RayTexImageAlpM2;
				layout (location = 1,r8) uniform restrict image3D _RayTexImageAlpM3;
				layout (location = 3,r8) uniform restrict image3D _RayTexImageAlpM4;
				layout (location = 5,r8) uniform restrict image3D _RayTexImageAlpM5;
				layout (location = 6,r8) uniform restrict image3D _RayTexImageAlpM6;
				layout (location = 7,r8) uniform restrict image3D _RayTexImageAlpM7;
				uniform sampler3D _RayTexSkip;
				uniform sampler3D _RayTexSkipHalf;
				uniform sampler3D _RayTexSkipQuarter;
				uniform sampler3D _RayTexDistance;
				uniform sampler2D Newness;
				uniform sampler2D BufferDepth;
				uniform sampler2D LastBufferDepth;
				uniform float Z;
				in float3 INworldPos;
				in float3 INPworldPos;
				in float3 INworldNor;
				in float4 INscreenPos;
				in float4 INPscreenPos;
				in float2 INuv;
				
				void main(){

					vec3 worldPos = INworldPos;
					vec3 worldNor = normalize(INworldNor);

					float4 Final = vec4(0.0);
					//Final.rgb = vec3(0,0.1,1)*40;
					//Final.rgb = vec3(0,0.3,1)*1;
					//Final.rgb = vec3(1,0.2,0)*1;
					//Final.rgb = vec3(0,-1,-1.5)*0.4;


					vec3 voxelPos = INworldPos;
					voxelPos = (_LightFieldWorldToLocal*vec4(voxelPos,1)).xyz;
					voxelPos += 0.5;
					voxelPos *= _LightFieldTexSize;

					ivec3 voxelPosInt = ivec3((voxelPos));


					imageStore(_RayTexImage, voxelPosInt, Final);
					imageStore(_RayTexImageM2, ivec3(voxelPos/2), Final);


					voxelPos = INworldPos;
					voxelPos = (_LightFieldWorldToLocal*float4(voxelPos,1)).xyz;
					voxelPos+=0.5;
					voxelPos*=_LightFieldDistTexSize;
					voxelPos.z+=1;//Push the balls above ground plane voxels


					voxelPosInt = ivec3(floor(voxelPos));


					imageStore(_RayTexImageAlpM2, voxelPosInt/2, float4(voxelPhase));
					imageStore(_RayTexImageAlpM3, voxelPosInt/4, float4(voxelPhase));
					imageStore(_RayTexImageAlpM4, voxelPosInt/8, float4(voxelPhase));
					imageStore(_RayTexImageAlpM5, voxelPosInt/16, float4(voxelPhase));
					imageStore(_RayTexImageAlpM6, voxelPosInt/32, float4(voxelPhase));
					imageStore(_RayTexImageAlpM7, voxelPosInt/64, float4(voxelPhase));
				}
			]]></Fragment>
		</Pass>
	</Passes>
</Shader>
