<Shader Name="Standard">
	<Uniforms>
		<Texture Name="AlbedoTexture" Uniform="AlbedoTexture"/>
		<Vector4 Name="AlbedoColor" Uniform="AlbedoColor"/>
		<Texture Name="NormalTexture" Uniform="NormalTexture" IsLinear="True"/>
		<Float Name="Roughness" Uniform="Roughness"/>
		<Float Name="Metalness" Uniform="Metalness"/>
		
		<Texture Name="EmissionTexture" Uniform="EmissionTexture"/>
		<Vector4 Name="EmissionColor" Uniform="EmissionColor" IsLinear="True"/>
		<Float Name="Strength" Uniform="Strength"/>
		
		<Texture Name="SSS_ColorTexture" Uniform="SSS_ColorTexture"/>
		<Vector4 Name="SSS_ColorColor" Uniform="SSS_ColorColor"/>
		<Float Name="SSS_Amount" Uniform="SSS_Amount"/>
		<Float Name="SSS_Thickness" Uniform="SSS_Thickness"/>
	</Uniforms>
	<Passes>
		<Pass Name="Deferred">
			<Vertex>
				#version 430
				#define VERTEX_SHADER
				#include "HLSL"
				#include "Random"
				 
				in vec3 position;
				in vec3 normal;
				out vec4 INscreenPos;
				out vec3 INworldPos;
				out vec4 INPscreenPos;
				out vec3 INworldNor;

				void main(){
					vec3 movpos = position;
					
					
					//Moving blocks...yeah I know...
					if (abs(max(movpos.x,max(movpos.y,movpos.z)))==1.0)
						movpos.x += sin(Time.x/2)*4;
					
					
					CalcPos(movpos);
					INworldPos = (_LocalToWorld*vec4(movpos, 1.0)).xyz;
					INscreenPos=CalcPos(movpos);
					INPscreenPos = _WorldToView_Past*_LocalToWorld_Past*vec4(movpos, 1.0);
					INworldNor = normalize((_LocalToWorldNormal*vec3(normal)).xyz);
					
					//Temporal Anti-aliasing Jitter
					float time = Time.w;//mod(Time.w,4);
					gl_Position.xy += (((vec2(Random(time),Random(time-138.4))))-0.5)*2*(ScreenSize.zw)*gl_Position.w;
				}
			</Vertex>
			<Fragment><![CDATA[
				#version 330 core
				#include "HLSL"
				#line 27
				in vec4 INscreenPos;
				in vec3 INworldPos;
				in vec4 INPscreenPos;
				in vec3 INworldNor;
				
				
				uniform vec4 AlbedoColor;
				uniform sampler2D AlbedoTexture;
				uniform vec4 EmissionColor;
				uniform float Strength;
				uniform float Roughness;
				
				layout(location = 0) out vec3 Normals;
				layout(location = 1) out vec2 MotionBuffer;
				
				void main(){
					vec3 worldNor = normalize(INworldNor);
					
					if (!gl_FrontFacing)
						worldNor*=-1;
					
					Normals = worldNor*0.5+0.5;
					
					
					vec3 screenPos = (INscreenPos.xyz/INscreenPos.w);
					vec3 PscreenPos = (INPscreenPos.xyz/INPscreenPos.w);
					
					MotionBuffer = (screenPos.xy-PscreenPos.xy)/2.0;
				}
			]]></Fragment>
		</Pass>
		<Pass Name="VoxelizeDetail">
			<Vertex><![CDATA[
				#version 330 core
				#define VERTEX_SHADER
				#include "HLSL"
				 
				in vec3 position;
				in vec3 normal;
				out vec3 INVworldPos;
				out vec3 INVworldNor;
				uniform float Strength;

				void main(){
					vec3 movpos = position;
					
					if (abs(max(movpos.x,max(movpos.y,movpos.z)))==1.0)
						movpos.x += sin(Time.x/2)*4;
					
					INVworldPos = (_LocalToWorld*vec4(movpos, 1.0)).xyz;
					INVworldNor = normalize((_LocalToWorldNormal*normal).xyz);
					gl_Position = _LocalToWorld*vec4(movpos,1.0);
				}
			]]></Vertex>
			<Fragment><![CDATA[
				#version 430 core
				#include "HLSL"
				#include "LightFields"
				#line 48
				layout (location = 0,r8) uniform restrict image3D _RayTexImageAlp;
				
				in vec3 INworldPos;
				in vec3 INworldNor;
				
				void main(){
					//Output the highest resolution voxel collision data
					vec3 voxelPos = INworldPos;
					voxelPos = (_LightFieldWorldToLocal*vec4(voxelPos,1)).xyz;
					voxelPos+=0.5;
					voxelPos*=_LightFieldDistTexSize;
					
					
					imageStore(_RayTexImageAlp, ivec3(voxelPos), vec4(voxelPhase));
				}
			]]></Fragment>
			<Geometry><![CDATA[
				#version 430

				uniform vec3 _LightFieldTexSize;
				uniform mat4 _LightFieldWorldToLocal;
				layout (triangles) in;
				layout (triangle_strip, max_vertices = 3) out;
				

				in vec3 INVworldPos[];
				in vec3 INVworldNor[];

				out vec3 INworldPos;
				out vec3 INworldNor;
				uniform mat4 _WorldToView;
				uniform mat4 _LocalToWorld;
				

				float triarea(vec2 p1, vec2 p2, vec2 p3){
					return abs(((p2.x - p1.x)*(p3.y - p1.y) - (p3.x - p1.x)*(p2.y - p1.y))/2.0);
				}
				void main(){
					int i;
					
					float XAxisarea = triarea(gl_in[0].gl_Position.yz,gl_in[1].gl_Position.yz,gl_in[2].gl_Position.yz);
					float YAxisarea = triarea(gl_in[0].gl_Position.xz,gl_in[1].gl_Position.xz,gl_in[2].gl_Position.xz);
					float ZAxisarea = triarea(gl_in[0].gl_Position.xy,gl_in[1].gl_Position.xy,gl_in[2].gl_Position.xy);
					
					vec3 tricenter = (gl_in[0].gl_Position.xyz+gl_in[1].gl_Position.xyz+gl_in[2].gl_Position.xyz)*0.333;
					for(i = 0; i < gl_in.length(); i++){
					
						gl_Position = gl_in[i].gl_Position;
						INworldPos = INVworldPos[i];
						INworldNor = INVworldNor[i];
						
						gl_Position = vec4((((_LightFieldWorldToLocal*vec4(gl_Position.xyz,1))).xyz*2.f).xyz,1.0);
						gl_Position.xyz = gl_Position.xyz;
						
						if (XAxisarea>=YAxisarea&&XAxisarea>=ZAxisarea){
							gl_Position.xyz = gl_Position.yzx;
							gl_Position.y*=(_LightFieldTexSize.z/_LightFieldTexSize.y);
						}
						if (YAxisarea>=XAxisarea&&YAxisarea>=ZAxisarea){
							gl_Position.xyz = gl_Position.xzy;
							gl_Position.y*=(_LightFieldTexSize.z/_LightFieldTexSize.y);
						}
						EmitVertex();
					}
					EndPrimitive();
				}
			]]></Geometry>
		</Pass>
	
	
	
		<Pass Name="VoxelizeMidAndColor">
			<Vertex><![CDATA[
				#version 430
				#define VERTEX_SHADER
				#include "HLSL"
				 
				in vec3 position;
				in vec3 normal;
				in vec2 uv;
				out vec3 INVworldPos;
				out vec3 INVPworldPos;
				out vec3 INVworldNor;
				out vec4 INVscreenPos;
				out vec4 INVPscreenPos;
				out vec2 INVuv;
				uniform float Strength;



				void main(){
					vec3 movpos = position;
					
					
					if (abs(max(movpos.x,max(movpos.y,movpos.z)))==1.0)
						movpos.x += sin(Time.x/2)*4;
					
					
					
					INVworldPos = (_LocalToWorld*vec4(movpos, 1.0)).xyz;
					INVscreenPos=_LocalToWorld*vec4(movpos, 1.0);
					gl_Position = _LocalToWorld*vec4(movpos,1.0);
					INVPscreenPos = _LocalToWorld_Past*vec4(movpos, 1.0);
					INVworldNor = normalize((_LocalToWorldNormal*normal).xyz);
					INVuv = uv;
					
					
					INVPworldPos = (_LocalToWorld_Past*vec4(movpos, 1.0)).xyz;
				}
			]]></Vertex>
			<Fragment><![CDATA[
				#version 430 core
				#include "HLSL"
				#line 601
				uniform vec4 AlbedoColor;
				uniform vec4 EmissionColor;
				uniform float Strength;
				uniform sampler2D AlbedoTexture;
				uniform sampler2D TempLightmapStopgap;
				uniform sampler3D _RayTex;
				
				//Way too high memory usage...don't even use A anymore XD
				layout (location = 0,rgba16f) uniform restrict image3D _RayTexImage;
				layout (location = 2,rgba16f) uniform restrict image3D _RayTexImageM2;
				
				layout (location = 4,r8) uniform restrict image3D _RayTexImageAlpM2;
				layout (location = 1,r8) uniform restrict image3D _RayTexImageAlpM3;
				layout (location = 3,r8) uniform restrict image3D _RayTexImageAlpM4;
				layout (location = 5,r8) uniform restrict image3D _RayTexImageAlpM5;
				layout (location = 6,r8) uniform restrict image3D _RayTexImageAlpM6;
				layout (location = 7,r8) uniform restrict image3D _RayTexImageAlpM7;
				uniform sampler3D _RayTexSkip;
				uniform sampler3D _RayTexSkipHalf;
				uniform sampler3D _RayTexSkipQuarter;
				uniform sampler3D _RayTexDistance;
				uniform sampler2D Newness;
				uniform sampler2D BufferDepth;
				uniform sampler2D LastBufferDepth;
				uniform float Z;
				in vec3 INworldPos;
				in vec3 INPworldPos;
				in vec3 INworldNor;
				in vec2 INuv;
				
				
				
				#define RaycastCombined
				#define LowQualityIS
				#define SingleDirect
				#include "LightFields"
				void main(){
					
					//Outputs the voxel collision data and color data
					
					//First output colors{
					vec4 color = vec4(1.0,1.0,1.0,1.0);
					
					vec3 worldPos = INworldPos;
					vec3 worldNor = normalize(INworldNor);

					vec2 flipuv = INuv;
					flipuv.y = 1.0-flipuv.y;

					//Sample the lightmap for diffuse bounces
					vec4 lightmap = textureLod(TempLightmapStopgap,flipuv,2);
					vec3 collighting = lightmap.rgb/max(0.001,lightmap.a);

					//Uncomment for GI calculation to occure in here
					/*{
						worldNor = normalize(INworldNor);
						vec2 castrand = Time.xx;//round(frac(Time.x/10))*(worldPos.xy+worldPos.z*worldPos.x)+Time.xx;

						if (AlbedoColor.a<1)worldNor*=-1;	


						vec3 binor = cross(worldNor,normalize(vec3(1,1,1)));
						vec3 tang = cross(worldNor,binor);


						mat3 rot = transpose(mat3(binor,tang,worldNor));

						float GISteps = 4;
						vec3 inddiflighting = vec3(0.0);
						if (dot((_CameraPos-worldPos),worldNor)<=0)
							worldNor*=-1.0;
						else
							inddiflighting = RaycastGI(int(1),worldPos,rot,0).xyz/3.1415;
						vec3 diflighting = vec3(0.0);
						for(int i = ISTypes.length()-1;i>=0;i--)
							diflighting.xyz += RaycastIS_Sphere(int(1),0,2,worldPos,worldNor,ISPositions[i],ISSizes1[i].r,0);

						collighting.xyz = diflighting+inddiflighting;
					}*/
					
					
					vec4 Final = vec4(collighting+EmissionColor.rgb*Strength,1.0);



					vec3 voxelPos = INworldPos;
					voxelPos = (_LightFieldWorldToLocal*vec4(voxelPos,1)).xyz;
					voxelPos += 0.5;
					voxelPos *= _LightFieldTexSize;
					
					ivec3 voxelPosInt = ivec3(voxelPos);
					
					//Output highest color resolution
					imageStore(_RayTexImage, voxelPosInt, Final);
					
					
					//Next resolution is for GI only, so ignore importance samplers
					bool isIS = false;
					if (true){
						for(int i = 0;i<ISTypes.length();i++){
							if (ISTypes[i]==ISSphere){
								if (dot(worldPos-ISPositions[i],worldPos-ISPositions[i])<pow(ISSizes1[i].x+0.2,2))
									isIS = true;
							}
						}
					}
					if (isIS){
						Final.rgb *= 0;	
					}
					//Ouput second color resolution
					imageStore(_RayTexImageM2, ivec3(voxelPos/2), Final);

					//}
					//Next output the other resolutions of voxel collision data

					voxelPos = INworldPos;
					voxelPos = (_LightFieldWorldToLocal*vec4(voxelPos,1)).xyz;
					voxelPos += 0.5;
					voxelPos *= _LightFieldDistTexSize;
					
					voxelPosInt = ivec3(voxelPos);
					
					
					imageStore(_RayTexImageAlpM2, voxelPosInt/2, vec4(voxelPhase));
					imageStore(_RayTexImageAlpM3, voxelPosInt/4, vec4(voxelPhase));
					imageStore(_RayTexImageAlpM4, voxelPosInt/8, vec4(voxelPhase));
					imageStore(_RayTexImageAlpM5, voxelPosInt/16, vec4(voxelPhase));
					imageStore(_RayTexImageAlpM6, voxelPosInt/32, vec4(voxelPhase));
					imageStore(_RayTexImageAlpM7, voxelPosInt/64, vec4(voxelPhase));
				}
			]]></Fragment>
			<Geometry><![CDATA[
				#version 430

				uniform vec3 _LightFieldTexSize;
				uniform mat4 _LightFieldWorldToLocal;
				layout (triangles) in;
				layout (triangle_strip, max_vertices = 3) out;
				

				in vec3 INVworldPos[];
				in vec3 INVPworldPos[];
				in vec3 INVworldNor[];
				in vec2 INVuv[];

				out vec3 INworldPos;
				out vec3 INPworldPos;
				out vec3 INworldNor;
				out vec2 INuv;
				uniform mat4 _WorldToView;
				uniform mat4 _LocalToWorld;
				

				float triarea(vec2 p1, vec2 p2, vec2 p3){
					return abs(((p2.x - p1.x)*(p3.y - p1.y) - (p3.x - p1.x)*(p2.y - p1.y))/2.0);
				}
				void main(){
					int i;
					
					float XAxisarea = triarea(gl_in[0].gl_Position.yz,gl_in[1].gl_Position.yz,gl_in[2].gl_Position.yz);
					float YAxisarea = triarea(gl_in[0].gl_Position.xz,gl_in[1].gl_Position.xz,gl_in[2].gl_Position.xz);
					float ZAxisarea = triarea(gl_in[0].gl_Position.xy,gl_in[1].gl_Position.xy,gl_in[2].gl_Position.xy);
					
					vec3 tricenter = (gl_in[0].gl_Position.xyz+gl_in[1].gl_Position.xyz+gl_in[2].gl_Position.xyz)*0.333;
					for(i = 0; i < gl_in.length(); i++){
					
						gl_Position = gl_in[i].gl_Position;
						INworldPos = INVworldPos[i];
						INPworldPos = INVPworldPos[i];
						INworldNor = INVworldNor[i];
						INuv = INVuv[i];


						gl_Position = vec4((((_LightFieldWorldToLocal*vec4(gl_Position.xyz,1))).xyz*2.f).xyz,1.0);
						gl_Position.xyz = gl_Position.xyz;
						
						
						if (XAxisarea>=YAxisarea&&XAxisarea>=ZAxisarea){
							gl_Position.xyz = gl_Position.yzx;
							gl_Position.y*=(_LightFieldTexSize.z/_LightFieldTexSize.y);
						}
						if (YAxisarea>=XAxisarea&&YAxisarea>=ZAxisarea){
							gl_Position.xyz = gl_Position.xzy;
							gl_Position.y*=(_LightFieldTexSize.z/_LightFieldTexSize.y);
						}
						EmitVertex();
					}
					EndPrimitive();
				}
			]]></Geometry>
		</Pass>	

		<Pass Name="LightmapGI">
			<Vertex><![CDATA[
				#version 330 core
				#define VERTEX_SHADER
				#include "HLSL"
				#pragma optionNV (unroll all)
				 
				in vec3 position;
				in vec3 normal;
				in vec2 uv;
				out vec3 INworldPos;
				out vec3 INworldNor;
				out vec3 INworldTan;
				out vec3 INworldBi;
				out vec4 INscreenPos;
				out vec4 INPscreenPos;
				out vec2 INuv;
				uniform float Strength;



				void main(){
					vec3 movpos = position;
					
					INworldPos = (_LocalToWorld*vec4(movpos, 1.0)).xyz;
					INscreenPos=CalcPos(movpos);
					INPscreenPos = _WorldToView_Past*_LocalToWorld_Past*vec4(movpos, 1.0);
					INscreenPos = _WorldToView*_LocalToWorld*vec4(movpos, 1.0);
					INworldNor = normalize((_LocalToWorldNormal*vec3(normal)).xyz);
					
					INworldBi = cross(INworldNor,normalize(vec3(1,1,1)));
					INworldTan = cross(INworldNor,INworldBi);
					INuv = uv;
					
					gl_Position.xy = vec2(uv.x,uv.y)*2.0-1.0;
					gl_Position.y *= -1.0;
					gl_Position.zw = vec2(1.0);
				}
			]]></Vertex>
			<Fragment><![CDATA[
				#version 430 core
				#include "HLSL"
				
				#define SingleDirect
				#include "LightFields"
				#line 59
				uniform vec4 AlbedoColor;
				uniform sampler2D AlbedoTexture;
				uniform vec4 EmissionColor;
				uniform float Strength;
				uniform float Roughness;
				uniform sampler2D TempLightmapStopgap;
				
				uniform usampler2D _LightFieldBits;
				
				uniform float Layer2;
				in vec3 INworldPos;
				in vec3 INworldNor;
				in vec3 INworldTan;
				in vec3 INworldBi;
				in vec4 INscreenPos;
				in vec4 INPscreenPos;
				in vec2 INuv;
				
				layout(location = 0) out vec4 Final;
				void main(){
					#line 530
					
					vec3 worldPos = INworldPos;
					vec3 worldNor = normalize(INworldNor);
					
					mat3 rot = mat3(normalize(INworldBi),normalize(INworldTan),(worldNor));
					
					
					
					
					vec3 diflighting = vec3(0,0,0);

					if (dot((_CameraPos-worldPos),worldNor)<=0)
						worldNor*=-1.0;
					else
						diflighting = RaycastGI(int(10),worldPos,rot,0).xyz/3.1415;


					for(int i = ISTypes.length()-1;i>=0;i--)
						diflighting.xyz += RaycastIS_Sphere(int(1),0,2,worldPos,worldNor,ISPositions[i],ISSizes1[i].r,0);
					
					


					Final = vec4(diflighting.rgb,1.0);
				}
			]]></Fragment>
		</Pass>
		<Pass Name="Depth">
			<Vertex>
				#version 430
				#define VERTEX_SHADER
				#include "HLSL"
				#include "Random"
				 
				in vec3 position;
				in vec3 normal;

				void main(){
					vec3 movpos = position;
					
					if (abs(max(movpos.x,max(movpos.y,movpos.z)))==1.0)
						movpos.x += sin(Time.x/2)*4;
					
					CalcPos(movpos);
					float time = Time.w;//mod(Time.w,4);//(mod(Time.w,2)==1)?1:0;
					gl_Position.xy += (((vec2(Random(time),Random(time-138.4))))-0.5)*2*(ScreenSize.zw)*gl_Position.w;
				}
			</Vertex>
			<Fragment><![CDATA[
				#version 330 core
				#include "HLSL"
				#line 27
				
				void main(){
				}
			]]></Fragment>
		</Pass>	
		<Pass Name="Emission">
			<Vertex>
				#version 430
				#define VERTEX_SHADER
				#include "HLSL"
				#include "Random"
				 
				in vec3 position;

				void main(){
					CalcPos(position);
					float time = Time.w;//mod(Time.w,4);
					gl_Position.xy += (((vec2(Random(time),Random(time-138.4))))-0.5)*2*(ScreenSize.zw)*gl_Position.w;
				}
			</Vertex>
			<Fragment><![CDATA[
				#version 330 core
				#include "HLSL"
				#line 27
				
				
				uniform vec4 AlbedoColor;
				uniform sampler2D AlbedoTexture;
				uniform vec4 EmissionColor;
				uniform float Strength;
				uniform float Roughness;
				
				layout(location = 0) out vec4 Lighting;
				
				void main(){
					Lighting = vec4(pow(EmissionColor.xyz,vec3(1.0))*Strength,1.0);
				}
			]]></Fragment>
		</Pass>
		<Pass Name="Forward">
			<Vertex>
				#version 430
				#define VERTEX_SHADER
				#include "HLSL"
				#include "Random"
				 
				in vec3 position;
				in vec3 normal;
				in vec2 uv;
				out vec4 INscreenPos;
				out vec3 INworldPos;
				out vec4 INPscreenPos;
				out vec3 INworldNor;
				out vec2 INuv;

				void main(){
					vec3 movpos = position;
					
					
					if (abs(max(movpos.x,max(movpos.y,movpos.z)))==1.0)
						movpos.x += sin(Time.x/2)*4;
					
					
					CalcPos(movpos);
					INworldPos = (_LocalToWorld*vec4(movpos, 1.0)).xyz;
					INscreenPos=CalcPos(movpos);
					INPscreenPos = _WorldToView_Past*_LocalToWorld_Past*vec4(movpos, 1.0);
					INworldNor = normalize((_LocalToWorldNormal*vec3(normal)).xyz);
					float time = Time.w;//mod(Time.w,4);
					
					
					gl_Position.xy += (((vec2(Random(time),Random(time-138.4))))-0.5)*2*(ScreenSize.zw)*gl_Position.w;
					INuv = uv;
				}
			</Vertex>
			<Fragment><![CDATA[
				#version 430 core
				#include "HLSL"
				#line 27
				//out vec4 color;
				in vec4 INscreenPos;
				in vec3 INworldPos;
				in vec4 INPscreenPos;
				in vec3 INworldNor;
				in vec2 INuv;
				
				
				
				#define SingleGI
				#define HighQualityGI
				#define SingleDirect
				#define RaycastCombined
				//#define ScreenSpaceRaycastingPossible
					#include "LightFields"
				
				
				uniform vec4 AlbedoColor;
				uniform sampler2D AlbedoTexture;
				uniform vec4 EmissionColor;
				uniform float Strength;
				uniform float Roughness;
				
				
				layout(location = 0) out vec4 Lighting;
				layout(location = 1) out vec2 MotionBuffer;
				
				
				void main(){
					vec3 worldNor = normalize(INworldNor);
					vec3 worldPos = INworldPos;
					if (!gl_FrontFacing)
						worldNor*=-1;
					
					vec3 screenPos = (INscreenPos.xyz/INscreenPos.w+1.0)/2.0;
					vec3 PscreenPos = (INPscreenPos.xyz/INPscreenPos.w+1.0)/2.0;
					MotionBuffer = screenPos.xy-PscreenPos.xy;
					
					Lighting = vec4(pow(EmissionColor.xyz,vec3(1.0))*Strength,1.0);
					
					
					vec2 myuv = floor(screenPos.xy*ScreenSize.xy)*ScreenSize.zw;
					float castrandscal = myuv.x+myuv.y*237.38+frac(Time.x)*209;
					
					
					vec3 hitPos;
					
					int i = int(mod(float(int(Random(castrandscal*0.18)*float(ISTypes.length()))),float(ISTypes.length())));
						Lighting.xyz += RaycastIS_Sphere(1,0,1,worldPos.xyz,worldNor,ISPositions[i],ISSizes1[i].r,castrandscal)*ISTypes.length();
					
					vec3 binor = cross(worldNor,vec3(0.577350269,0.577350269,0.577350269));
					vec3 tang = cross(worldNor,binor);
					mat3 rot = (mat3(binor,tang,worldNor));
					Lighting.xyz += RaycastGI(1,worldPos.xyz,rot,castrandscal).xyz;
					
					
					vec3 worldView = normalize(worldPos.xyz-_CameraPos);		
					float fresnel = max(0.0,1.0-max(0,dot(worldNor,-worldView)));
					fresnel = fresnel*fresnel;
					fresnel = fresnel*fresnel;	
					{			
						vec3 reflNor = normalize(reflect(worldView,worldNor));	
						
						vec3 binor = cross(reflNor,normalize(vec3(1,1,1)));
						vec3 tang = cross(reflNor,binor);
						rot = (mat3(binor,tang,reflNor));
						}
					{
						float blurriness = 0.02;
						vec3 castdir = normalize(rot * (HemisphereCosineSample(Random(castrandscal),Random(castrandscal+2))*vec3(blurriness,blurriness,1.0)));
						vec3 speclighting = whatevercast(worldPos.xyz+worldNor*0.08,castdir,55,0).rgb;
						
						Lighting.xyz = lerp(Lighting.xyz,speclighting,fresnel);
						
					}
				}
			]]></Fragment>
		</Pass>
	</Passes>
</Shader>
