<Shader Name="GenerateAcceleration">
	<Uniforms>
		<Texture Name="Main Texture" Uniform="_MainTex"/>
	</Uniforms>
	<Passes>
		<Pass Name="Deferred">
			<Vertex>
				#version 330 core
				#define VERTEX_SHADER
				#include "HLSL"
				 
				in float3 position;
				in float2 uv;
				out float2 VinUV;

				void main(){
					gl_Position = float4(position,1);
					VinUV = position.xy*float2(0.5,0.5)+float2(0.5,0.5);
				}
			</Vertex>
			<Fragment><![CDATA[
				#version 330 core
				#include "HLSL"
				#line 24
				out uint color;
				in float3 inUV;

				uniform sampler3D _MainTex;
				uniform float voxelPhase;
				void main(){
					//Find the first free voxel, starting from the largest size and going to the smallest, storing the size
                    float3 uv =inUV;
					color = 64u;
					if (textureLod(_MainTex,uv,6).r!=voxelPhase)return;
					color = 32u;
					if (textureLod(_MainTex,uv,5).r!=voxelPhase)return;
					color = 16u;
					if (textureLod(_MainTex,uv,4).r!=voxelPhase)return;
					color = 8u;
					if (textureLod(_MainTex,uv,3).r!=voxelPhase)return;
					color = 4u;
					if (textureLod(_MainTex,uv,2).r!=voxelPhase)return;
					color = 2u;
					if (textureLod(_MainTex,uv,1).r!=voxelPhase)return;
					color = 1u;
					return;
				}
			]]></Fragment>
			<Geometry><![CDATA[
				#version 400 core
				
				layout (triangles) in;
				layout(invocations = 32) in;
				layout (triangle_strip, max_vertices = 32) out;
				

				in vec2 VinUV[];

				out vec3 inUV;
				uniform vec3 _LightFieldTexSize;
				
				void main()
				{
					//One shader invocation, render to every layer of the 3D texture
					for (int layer = 0;layer<8;layer+=1){
						for(int i = 0; i < gl_in.length(); i++){

							gl_Position = gl_in[i].gl_Position;
							gl_Layer = (gl_InvocationID)*8+layer;
							inUV = vec3(VinUV[i],(float((gl_InvocationID)*8+layer)+0.5)/(_LightFieldTexSize.z));
							EmitVertex();
						}
						EndPrimitive();
					}
				}
			]]></Geometry>
		</Pass>
	</Passes>
</Shader>

