﻿<Shader Name="LightingCompute">
	<Uniforms>
	</Uniforms>
	<Passes>
		<Pass Name="GlobalIllumination">
			<Compute><![CDATA[
				#version 430
				#include "HLSL"
				layout(local_size_x = 20, local_size_y = 20) in;
				
				layout (location = 0,rgba16f) uniform restrict image2D img_output;

				uniform sampler2D BufferDepth;
				uniform sampler2D LinearDepth;
				uniform sampler2D Normals;
				uniform sampler2D Albedo;
				
				
					#define SingleGI//Optimize code for taking 1 GI sample
					#define SingleDirect//Optimize code for taking 1 IS sample
					#define HighQualityGI//Allow more accurate GI intersections
					#define RaycastCombined//Define a function that takes a random IS or GI sample
					#define ScreenSpaceRaycastingPossible//Enable screenspace raycasting for more accurate small-scale intersection (Currently broken)
				#include "LightFields"


				const[PreCompile]uniform float NoiseStyle;
				const[PreCompile]uniform float _HQReflections;
				const[PreCompile]uniform float _NoiseStyle;
				const[PreCompile]uniform float _CastGI;
				const[PreCompile]uniform float _CastIS;
				const[PreCompile]uniform float _CastSpecular;
				void main() {
					
					
					ivec2 screenCoord = ivec2(gl_GlobalInvocationID.x,gl_GlobalInvocationID.y);
					//Normalized UVs (0-1)
					vec2 uv = (vec2(screenCoord)+0.5)*ScreenSize.zw;
					
					float dep = textureLod(BufferDepth,uv,0).r;
					if (dep==1.0)//If the sky, skip
						return;
					
					//Get position and normal
					vec3 worldNor = textureLod(Normals,uv,0).rgb*2.0-1.0;
					vec4 worldPosTemp = _InvWorldToView*vec4(vec3(uv,dep)*2.0-1.0,1.0);
					vec3 worldPos = worldPosTemp.xyz/=worldPosTemp.w;
					
					//Random value for each pixel
					float castrandscal = uv.x+uv.y*237.38+frac(Time.x)*209;
					
					vec4 pixel = vec4(0.0, 0.0, 0.0, 0.0);
					
					//Bounce ray/Other lights ray
					if (_CastGI==1.0){
						//Create rotation basis for hemisphere 
						//(so I can generate random directions on a hemisphere then rotate them to follow the normal)
						vec3 binor = cross(worldNor,vec3(0.577350269,0.577350269,0.577350269));
						vec3 tang = cross(worldNor,binor);
						mat3 rot = (mat3(binor,tang,worldNor));
						//Raycast!
						pixel.xyz += RaycastGI(1,worldPos,rot,castrandscal).xyz;
					}
					
					//Importance sample ray
					if (_CastIS==1.0){
						//Only sample 1 importance sampler, randomly
						if (_NoiseStyle==0){
							int i = int(float(int(Random(castrandscal*0.18)*float(ISTypes.length()))));//Likelyhood of Random() hitting 1.0 is so low I don't bother clamping...
							//And yeah the int(float(int(x))) is pointless but it runs faster on my gpu... :/
							pixel.xyz += RaycastIS_Sphere(int(ISSamples[i]*8),0,1,worldPos,worldNor,ISPositions[i],ISSizes1[i].r,castrandscal)*ISTypes.length();
						}else 
							
						//Only sample 1 importance sampler, evenly (so it looks almost the same as if I were casting all)
						if (_NoiseStyle==1){
							//Pick a random light offset per pixel, then each frame sample the next light in the cycle, guarantees that after the 
							//same number of frames as there are lights all will have been samples and combined with the temporal supersampling
							int i = int(mod(float(int(Random(uv.x+uv.y*237.38+22.83)*float(ISTypes.length()))+int(Time.w)),float(ISTypes.length())));
							pixel.xyz += RaycastIS_Sphere(int(ISSamples[i]*8),0,1,worldPos,worldNor,ISPositions[i],ISSizes1[i].r,castrandscal)*ISTypes.length();
						}else 
							
						//Sample all
						if (_NoiseStyle==2){
							for(int i = ISTypes.length()-1;i>=0;i--)
								pixel.xyz += RaycastIS_Sphere(int(ISSamples[i]*8),0,1,worldPos,worldNor,ISPositions[i],ISSizes1[i].r,castrandscal);
						}
					}
					
					vec3 worldView = normalize(worldPos-_CameraPos);	

					//Lazy fresnel
					float fresnel = min(1.0,1.0-dot(worldNor,-worldView));
					fresnel = fresnel*fresnel;
					fresnel = fresnel*fresnel;
					
					//Reflection ray
					if (_CastSpecular==1.0)
					{
						vec3 reflNor = normalize(reflect(worldView,worldNor));	
						
						//Once again calculate rotation basis based on reflection ray
						vec3 binor = cross(reflNor,normalize(vec3(1,1,1)));
						vec3 tang = cross(reflNor,binor);
						mat3 rot = (mat3(binor,tang,reflNor));
						
						float glossiness = 0.04;
						
						//Yes my specular brdf is a squished hemisphere :P
						vec3 castdir = normalize(rot * (HemisphereCosineSample(Random(castrandscal),Random(castrandscal+2))*vec3(glossiness,glossiness,1.0)));
						
						vec3 speclighting;
						vec3 hitPos;
						if (_HQReflections==1.0)
							speclighting = raycast(0,worldPos+worldNor*0.04,castdir,false,55,hitPos).rgb;
						else
							speclighting = conecastish(worldPos+worldNor*0.06,castdir,55,0).rgb;
						
						//Blend based on fresnel - I'm thinking of trying stochastic sampling at some point as well (randomly picking either reflection or diffuse rays only based on fresnel)
						pixel.xyz = lerp(pixel.xyz,speclighting,fresnel);
					}
					imageStore(img_output, ivec2(screenCoord.xy), pixel);
				}
			]]></Compute>
		</Pass>
	</Passes>
</Shader>